// INTERFACE.CPP 
//
//
// PREFIX:
// IF_<functionname>  -> Interface function
//

#ifndef _WIN32
#include <string.h>
#endif

// HALF-LIFE 1 SPECIFIC HEADERS

#include <extdll.h>
#include <dllapi.h>
#include <h_export.h>
#include <meta_api.h>
#include <entity_state.h>
#include <sdk_util.h>
// END OF HL1 


///////////////////////////////
// ITM SPECIFIC HEADERS
#include "..\..\itm_shared\itm_shared.h"        // Set it all up here

#include "itm_interface_hl1.h"            // include proper header file for this interface
// ITM SPECIFIC HEADERS - END
///////////////////////////////

#define CSTRIKE_DLL     99
#define VALVE_DLL       98

// FUNCTIONS for Metamod
DLL_FUNCTIONS gFunctionTable;
DLL_FUNCTIONS gFunctionTable_post;
enginefuncs_t g_engfuncs;
globalvars_t *gpGlobals;

// Half-Life 1 Vector g_vecZero
DLL_GLOBAL const Vector g_vecZero = Vector (0, 0, 0);

// Variables to make sure we keep track of the very basic data for clients
edict_t *clients[32];                   // 32 clients in HL1
edict_t *pHostEdict = NULL;             // Possible 'host edict' for listen servers
float welcome_time = 0.0;
bool welcome_sent = false;
float bot_check_time = 30.0;
int mod_id = CSTRIKE_DLL;


// START of Metamod stuff
enginefuncs_t meta_engfuncs;
gamedll_funcs_t *gpGamedllFuncs;
mutil_funcs_t *gpMetaUtilFuncs;
meta_globals_t *gpMetaGlobals;

// ITM Server commands
void ITM_ServerCommand (void);



// METAMOD INSTALLMENT

META_FUNCTIONS gMetaFunctionTable = {
  NULL,					// pfnGetEntityAPI()
  NULL,					// pfnGetEntityAPI_Post()
  GetEntityAPI2,		// pfnGetEntityAPI2()
  GetEntityAPI2_Post,	// pfnGetEntityAPI2_Post()
  NULL,					// pfnGetNewDLLFunctions()
  NULL,					// pfnGetNewDLLFunctions_Post()
  GetEngineFunctions,	// pfnGetEngineFunctions()
  NULL,					// pfnGetEngineFunctions_Post()
};

plugin_info_t Plugin_info = {
   META_INTERFACE_VERSION,				// interface version
   "ITM-Botframe",	                    // plugin name
   "Alpha",							    // plugin version
   __DATE__,							// date of creation
   "Stefan Hendriks",					// plugin author
   "http://realbot.bots-united.com/",	// plugin URL
   "ITMBOT",							// plugin logtag
   PT_ANYTIME,						// when loadable <-- FIX
   PT_ANYTIME,							// when unloadable
};

C_DLLEXPORT int Meta_Query (char *ifvers, plugin_info_t ** pPlugInfo, mutil_funcs_t * pMetaUtilFuncs)
{
  // this function is the first function ever called by metamod in the plugin DLL. Its purpose
  // is for metamod to retrieve basic information about the plugin, such as its meta-interface
  // version, for ensuring compatibility with the current version of the running metamod.
    ITM_Log("META: Query\n");
  // keep track of the pointers to metamod function tables metamod gives us
  gpMetaUtilFuncs = pMetaUtilFuncs;
  *pPlugInfo = &Plugin_info;

  // check for interface version compatibility
  if (strcmp (ifvers, Plugin_info.ifvers) != 0)
  {
      int mmajor = 0, mminor = 0, pmajor = 0, pminor = 0;
      LOG_CONSOLE (PLID, 
          "%s: meta-interface version mismatch (metamod: %s, %s: %s)",
		   Plugin_info.name, ifvers, Plugin_info.name,
		   Plugin_info.ifvers);
      LOG_MESSAGE (PLID,
		   "%s: meta-interface version mismatch (metamod: %s, %s: %s)",
		   Plugin_info.name, ifvers, Plugin_info.name,
		   Plugin_info.ifvers);

      // if plugin has later interface version, it's incompatible (update metamod)
      sscanf (ifvers, "%d:%d", &mmajor, &mminor);
      sscanf (META_INTERFACE_VERSION, "%d:%d", &pmajor, &pminor);
      if ((pmajor > mmajor) || ((pmajor == mmajor) && (pminor > mminor)))
	  {
		  LOG_CONSOLE (PLID,
			  "metamod version is too old for this plugin; update metamod");
		  LOG_ERROR (PLID,
			  "metamod version is too old for this plugin; update metamod");

          ITM_Log("PLUGIN: Load, error #1\n");
		  return (FALSE);
	  }

      // if plugin has older major interface version, it's incompatible (update plugin)
      else if (pmajor < mmajor)
	  {
		  LOG_CONSOLE (PLID,
			  "metamod version is incompatible with this plugin; please find a newer version of this plugin");
		  LOG_ERROR (PLID,
			  "metamod version is incompatible with this plugin; please find a newer version of this plugin");
          ITM_Log("PLUGIN: Load, error #2\n");
		  return (FALSE);
	  }
  }

  SERVER_PRINT("PLUGIN: So far so good...\n");
  ITM_Log("PLUGIN: Load, so far so good\n");
  return (TRUE);		// tell metamod this plugin looks safe
}

C_DLLEXPORT int Meta_Attach (PLUG_LOADTIME now, META_FUNCTIONS * pFunctionTable,
                             meta_globals_t * pMGlobals, gamedll_funcs_t * pGamedllFuncs)
{
    ITM_Log("META: Attach\n");
  // this function is called when metamod attempts to load the plugin. Since it's the place
  // where we can tell if the plugin will be allowed to run or not, we wait until here to make
  // our initialization stuff, like registering CVARs and dedicated server commands.

  // are we allowed to load this plugin now ?
  if (now > Plugin_info.loadable)
  {
      LOG_CONSOLE (PLID,
		  "%s: plugin NOT attaching (can't load plugin right now)",
		  Plugin_info.name);
      LOG_ERROR (PLID,
		  "%s: plugin NOT attaching (can't load plugin right now)",
		  Plugin_info.name);      
      ITM_Log("PLUGIN: Load, error #3\n");
      return (FALSE);		// returning FALSE prevents metamod from attaching this plugin
  }

  // keep track of the pointers to engine function tables metamod gives us
  gpMetaGlobals = pMGlobals;
  memcpy (pFunctionTable, &gMetaFunctionTable, sizeof (META_FUNCTIONS));
  gpGamedllFuncs = pGamedllFuncs;

  // print a message to notify about plugin attaching
  LOG_CONSOLE (PLID, "%s: plugin attaching", Plugin_info.name);
  LOG_MESSAGE (PLID, "%s: plugin attaching", Plugin_info.name);

  // ask the engine to register the server commands this plugin uses
  REG_SVR_COMMAND ("itm", ITM_ServerCommand);

  // Notify user that 'realbot' command is regged
  LOG_CONSOLE (PLID, "itm - command prefix is now reserved.");
  LOG_MESSAGE (PLID, "itm - command prefix is now reserved.");
  SERVER_PRINT("PLUGIN: YEAH\n");
  ITM_Log("PLUGIN: Load, YEAH\n");
  return (TRUE);		// returning TRUE enables metamod to attach this plugin
}

C_DLLEXPORT int Meta_Detach (PLUG_LOADTIME now, PL_UNLOAD_REASON reason)
{
  ITM_Log("META: Detach\n");
  // this function is called when metamod unloads the plugin. A basic check is made in order
  // to prevent unloading the plugin if its processing should not be interrupted.

  // is metamod allowed to unload the plugin ?
  if ((now > Plugin_info.unloadable) && (reason != PNL_CMD_FORCED))
  {
      LOG_CONSOLE (PLID,
		  "%s: plugin NOT detaching (can't unload plugin right now)",
		  Plugin_info.name);
      LOG_ERROR (PLID,
		  "%s: plugin NOT detaching (can't unload plugin right now)",
		  Plugin_info.name);
      return (FALSE);		// returning FALSE prevents metamod from unloading this plugin
  }

//  free(message);
  return (TRUE);		// returning TRUE enables metamod to unload this plugin
}
// END of Metamod stuff


void (*botMsgFunction) (void *, int) = NULL;
void (*botMsgEndFunction) (void *, int) = NULL;
int botMsgIndex;

void pfnChangeLevel (char *s1, char *s2)
{
    /*
  // kick any bot off of the server after time/frag limit...
    for (int index = 0; index < MAX_HL1_CLIENTS; index++)
    {
		if (ITM_Bots[index].bUsed)	// is this slot used?
		{
			char cmd[40];

			sprintf (cmd, "kick \"%s\"\n", bots[index].name);

			bots[index].respawn_state = RESPAWN_NEED_TO_RESPAWN;

			SERVER_COMMAND (cmd);	// kick the bot using (kick "name")
		}
    }

  if (Game.bEngineDebug)
	  rblog("ENGINE: pfnChangeLevel()\n");
      */

  RETURN_META (MRES_IGNORED);
}

edict_t *pfnFindEntityByString (edict_t * pEdictStartSearchAfter, const char *pszField,
		       const char *pszValue)
{

    // Counter-Strike - New Round Started
    if (strcmp (pszValue, "info_map_parameters") == 0)
    {
        // CS 1.5 only        
    }

  
  RETURN_META_VALUE (MRES_IGNORED, NULL);
}

void pfnRemoveEntity (edict_t * e)
{

  RETURN_META (MRES_IGNORED);
}

void pfnClientCommand (edict_t * pEdict, char *szFmt, ...)
{
  // new?
  if (pEdict->v.flags & (FL_FAKECLIENT))
    RETURN_META (MRES_SUPERCEDE);
  
  RETURN_META (MRES_IGNORED);
}

void pfnMessageBegin (int msg_dest, int msg_type, const float *pOrigin,
		 edict_t * ed)
{
  
  if (gpGlobals->deathmatch)
    {
      int index = -1;

	  // Fix this up for CS 1.6 weaponlists
	  // 01/07/04 - Stefan - Thanks to Whistler for pointing this out!
      if (msg_type == GET_USER_MSG_ID (PLID, "WeaponList", NULL)) 
         botMsgFunction = BotClient_CS_WeaponList; 

      if (ed)
	  {
		  index = UTIL_GetBotIndex (ed);
		  		  
		  // Note by stefan: This needs to get cleaned up
		  if (msg_type == GET_USER_MSG_ID(PLID, "SayText", NULL))
		  {
			  //BotMsgFunction = BotClient_CS_SayText;
			  /*
			  SERVER_PRINT("Recieved a message for : ");
			  SERVER_PRINT(STRING(ed->v.netname));
			  SERVER_PRINT("\n");
			  */
		  }		  

		  // is this message for a bot?
		  if (index != -1)
		  {
			  botMsgFunction = NULL;	// no msg function until known otherwise
			  botMsgEndFunction = NULL;	// no msg end function until known otherwise
			  botMsgIndex = index;	// index of bot receiving message

			  if (mod_id == VALVE_DLL)
			  {
				  if (msg_type == GET_USER_MSG_ID (PLID, "WeaponList", NULL))
					  botMsgFunction = BotClient_Valve_WeaponList;
				  else if (msg_type ==
					  GET_USER_MSG_ID (PLID, "CurWeapon", NULL))
					  botMsgFunction = BotClient_Valve_CurrentWeapon;
				  else if (msg_type == GET_USER_MSG_ID (PLID, "AmmoX", NULL))
					  botMsgFunction = BotClient_Valve_AmmoX;
				  else if (msg_type ==
					  GET_USER_MSG_ID (PLID, "AmmoPickup", NULL))
					  botMsgFunction = BotClient_Valve_AmmoPickup;
				  else if (msg_type ==
					  GET_USER_MSG_ID (PLID, "WeapPickup", NULL))
					  botMsgFunction = BotClient_Valve_WeaponPickup;
				  else if (msg_type ==
					  GET_USER_MSG_ID (PLID, "ItemPickup", NULL))
					  botMsgFunction = BotClient_Valve_ItemPickup;
				  else if (msg_type == GET_USER_MSG_ID (PLID, "Health", NULL))
					  botMsgFunction = BotClient_Valve_Health;
				  else if (msg_type ==
					  GET_USER_MSG_ID (PLID, "Battery", NULL))
					  botMsgFunction = BotClient_Valve_Battery;
				  else if (msg_type ==
					  GET_USER_MSG_ID (PLID, "DeathMsg", NULL))
					  botMsgFunction = BotClient_Valve_Damage;
				  else if (msg_type ==
					  GET_USER_MSG_ID (PLID, "ScreenFade", NULL))
					  botMsgFunction = BotClient_Valve_ScreenFade;
			  }
			  else if (mod_id == CSTRIKE_DLL)
			  {
				  if (msg_type == GET_USER_MSG_ID (PLID, "VGUIMenu", NULL))
					  botMsgFunction = BotClient_CS_VGUI;
				  else if (msg_type ==
					  GET_USER_MSG_ID (PLID, "ShowMenu", NULL))
					  botMsgFunction = BotClient_CS_ShowMenu;
				  else if (msg_type ==
					  GET_USER_MSG_ID (PLID, "WeaponList", NULL))
				  {
					  botMsgFunction = BotClient_CS_WeaponList;
				  }
				  else if (msg_type ==
					  GET_USER_MSG_ID (PLID, "CurWeapon", NULL))
					  botMsgFunction = BotClient_CS_CurrentWeapon;
				  else if (msg_type == GET_USER_MSG_ID (PLID, "AmmoX", NULL))
					  botMsgFunction = BotClient_CS_AmmoX;
				  else if (msg_type ==
					  GET_USER_MSG_ID (PLID, "WeapPickup", NULL))
					  botMsgFunction = BotClient_CS_WeaponPickup;
				  else if (msg_type ==
					  GET_USER_MSG_ID (PLID, "AmmoPickup", NULL))
					  botMsgFunction = BotClient_CS_AmmoPickup;
				  else if (msg_type ==
					  GET_USER_MSG_ID (PLID, "ItemPickup", NULL))
					  botMsgFunction = BotClient_CS_ItemPickup;
				  else if (msg_type == GET_USER_MSG_ID (PLID, "Health", NULL))
					  botMsgFunction = BotClient_CS_Health;
				  else if (msg_type ==
					  GET_USER_MSG_ID (PLID, "Battery", NULL))
					  botMsgFunction = BotClient_CS_Battery;
				  else if (msg_type == GET_USER_MSG_ID (PLID, "Damage", NULL))
					  botMsgFunction = BotClient_CS_Damage;		  
				  else if (msg_type == GET_USER_MSG_ID (PLID, "StatusIcon", NULL))
				  {
					  BotClient_CS_StatusIcon (0, -1);	// clear state -- redo this -- BERKED
					  botMsgFunction = BotClient_CS_StatusIcon;
				  }
				  if (msg_type == GET_USER_MSG_ID (PLID, "SayText", NULL))		  
				  {
					  botMsgFunction = BotClient_CS_SayText;		
				  }
				  else if (msg_type == GET_USER_MSG_ID (PLID, "Money", NULL))
					  botMsgFunction = BotClient_CS_Money;
				  else if (msg_type ==
			   GET_USER_MSG_ID (PLID, "ScreenFade", NULL))
		    botMsgFunction = BotClient_CS_ScreenFade;
		}
	    }
	}
      else if (msg_dest == MSG_ALL)
	{
	  botMsgFunction = NULL;	// no msg function until known otherwise
	  botMsgIndex = -1;	// index of bot receiving message (none)

	  if (mod_id == VALVE_DLL)
	    {
	      if (msg_type == GET_USER_MSG_ID (PLID, "DeathMsg", NULL))
		botMsgFunction = BotClient_Valve_DeathMsg;
	    }
	  else if (mod_id == CSTRIKE_DLL)
	    {
	      if (msg_type == GET_USER_MSG_ID (PLID, "DeathMsg", NULL))
			  botMsgFunction = BotClient_CS_DeathMsg;
		  else if (msg_type == GET_USER_MSG_ID (PLID, "SayText", NULL))		  
			botMsgFunction = BotClient_CS_SayText;		  
	    }
	}
	    // STEFAN
      else if (msg_dest == MSG_SPEC)
      {
         botMsgFunction = NULL;  // no msg function until known otherwise
         botMsgIndex = -1;       // index of bot receiving message (none)

         if (mod_id == CSTRIKE_DLL)
         {
             if (msg_type == GET_USER_MSG_ID (PLID, "HLTV", NULL))
                 botMsgFunction = BotClient_CS_HLTV;
			 else if (msg_type == GET_USER_MSG_ID (PLID, "SayText", NULL))		  
				 botMsgFunction = BotClient_CS_SayText;		  
         }

      }
      // STEFAN END 
    }

  RETURN_META (MRES_IGNORED);
}


void pfnMessageEnd (void)
{
  if (gpGlobals->deathmatch)
  {
      if (botMsgEndFunction)
          (*botMsgEndFunction) (NULL, botMsgIndex);	// NULL indicated msg end

      // clear out the bot message function pointers...
      botMsgFunction = NULL;
      botMsgEndFunction = NULL;
  }
  
  RETURN_META (MRES_IGNORED);
}

void pfnWriteByte (int iValue)
{
  if (gpGlobals->deathmatch)
  {
      // if this message is for a bot, call the client message function...
      if (botMsgFunction)
          (*botMsgFunction) ((void *) &iValue, botMsgIndex);
  }

  RETURN_META (MRES_IGNORED);
} 

void pfnWriteChar (int iValue)
{
  if (gpGlobals->deathmatch)
  {
      // if this message is for a bot, call the client message function...
      if (botMsgFunction)
          (*botMsgFunction) ((void *) &iValue, botMsgIndex);
  }

  RETURN_META (MRES_IGNORED);
} 

void pfnWriteShort (int iValue)
{
  if (gpGlobals->deathmatch)
  {
      // if this message is for a bot, call the client message function...
      if (botMsgFunction)
          (*botMsgFunction) ((void *) &iValue, botMsgIndex);
  }

  RETURN_META (MRES_IGNORED);
} 

void pfnWriteLong (int iValue)
{
  if (gpGlobals->deathmatch)
  {
      // if this message is for a bot, call the client message function...
      if (botMsgFunction)
          (*botMsgFunction) ((void *) &iValue, botMsgIndex);
  }
  
  RETURN_META (MRES_IGNORED);
} 

void pfnWriteAngle (float flValue)
{
    if (gpGlobals->deathmatch)
    {
        // if this message is for a bot, call the client message function...
        if (botMsgFunction)
            (*botMsgFunction) ((void *) &flValue, botMsgIndex);
    }

  RETURN_META (MRES_IGNORED);
} 

void pfnWriteCoord (float flValue)
{
  if (gpGlobals->deathmatch)
  {
      // if this message is for a bot, call the client message function...
      if (botMsgFunction)
          (*botMsgFunction) ((void *) &flValue, botMsgIndex);
  }

  RETURN_META (MRES_IGNORED);
} 

void pfnWriteString (const char *sz)
{
  if (gpGlobals->deathmatch)
  {
      // Ditlew's Radio
  }
  
  // if this message is for a bot, call the client message function...
  if (botMsgFunction)
      (*botMsgFunction) ((void *) sz, botMsgIndex);
  

  RETURN_META (MRES_IGNORED);
} 

void
pfnWriteEntity (int iValue)
{
  if (gpGlobals->deathmatch)
    {
      // if this message is for a bot, call the client message function...
      if (botMsgFunction)
	(*botMsgFunction) ((void *) &iValue, botMsgIndex);
    }

  RETURN_META (MRES_IGNORED);
}

void pfnClientPrintf (edict_t * pEdict, PRINT_TYPE ptype, const char *szMsg)
{
  // prevent bots sending these kind of messages
  if (pEdict->v.flags & (FL_FAKECLIENT))
    RETURN_META (MRES_SUPERCEDE);
  RETURN_META (MRES_IGNORED);
}

const char *pfnCmd_Args (void)
{
  //if (isFakeClientCommand)
    //RETURN_META_VALUE (MRES_SUPERCEDE, &g_argv[0]);

  RETURN_META_VALUE (MRES_IGNORED, NULL);
}

const char *pfnCmd_Argv (int argc)
{
/*  if (isFakeClientCommand)

    {
      if (argc == 0)
	RETURN_META_VALUE (MRES_SUPERCEDE, &g_argv[64]);

      else if (argc == 1)
	RETURN_META_VALUE (MRES_SUPERCEDE, &g_argv[128]);

      else if (argc == 2)
	RETURN_META_VALUE (MRES_SUPERCEDE, &g_argv[192]);

      else
	RETURN_META_VALUE (MRES_SUPERCEDE, NULL);
    }
    */

  RETURN_META_VALUE (MRES_IGNORED, NULL);
}

int pfnCmd_Argc (void)
{
    /*
  if (isFakeClientCommand)
    RETURN_META_VALUE (MRES_SUPERCEDE, fake_arg_count);
    */
  RETURN_META_VALUE (MRES_IGNORED, 0);
}

void pfnSetClientMaxspeed (const edict_t * pEdict, float fNewMaxspeed)
{
  // Set client max speed (CS / All mods)
/*
  // Check if edict_t is a bot, then set maxspeed
  cBot *pPlayerBot = NULL;
  int index;

  for (index = 0; index < 32; index++)
    {
      if (bots[index].pEdict == pEdict)
	{
	  break;
	}
    }

  if (index < 32)
    pPlayerBot = (&bots[index]);

  if (pPlayerBot)
    pPlayerBot->f_max_speed = fNewMaxspeed;*/

  RETURN_META (MRES_IGNORED);
}

int pfnGetPlayerUserId (edict_t * e)
{
  if (gpGlobals->deathmatch)
    {
      //if (mod_id == GEARBOX_DLL)
      //{
      //   // is this edict a bot?
      //   if (UTIL_GetBotPointer( e ))
      //      return 0;  // don't return a valid index (so bot won't get kicked)
      //}
    }

  RETURN_META_VALUE (MRES_IGNORED, 0);
}

C_DLLEXPORT int GetEngineFunctions (enginefuncs_t * pengfuncsFromEngine,
		    int *interfaceVersion)
{
  meta_engfuncs.pfnChangeLevel = pfnChangeLevel;
  meta_engfuncs.pfnFindEntityByString = pfnFindEntityByString;
  meta_engfuncs.pfnRemoveEntity = pfnRemoveEntity;
  meta_engfuncs.pfnClientCommand = pfnClientCommand;
  meta_engfuncs.pfnMessageBegin = pfnMessageBegin;
  meta_engfuncs.pfnMessageEnd = pfnMessageEnd;
  meta_engfuncs.pfnWriteByte = pfnWriteByte;
  meta_engfuncs.pfnWriteChar = pfnWriteChar;
  meta_engfuncs.pfnWriteShort = pfnWriteShort;
  meta_engfuncs.pfnWriteLong = pfnWriteLong;
  meta_engfuncs.pfnWriteAngle = pfnWriteAngle;
  meta_engfuncs.pfnWriteCoord = pfnWriteCoord;
  meta_engfuncs.pfnWriteString = pfnWriteString;
  meta_engfuncs.pfnWriteEntity = pfnWriteEntity;
  meta_engfuncs.pfnClientPrintf = pfnClientPrintf;
  meta_engfuncs.pfnCmd_Args = pfnCmd_Args;
  meta_engfuncs.pfnCmd_Argv = pfnCmd_Argv;
  meta_engfuncs.pfnCmd_Argc = pfnCmd_Argc;
  meta_engfuncs.pfnSetClientMaxspeed = pfnSetClientMaxspeed;
  meta_engfuncs.pfnGetPlayerUserId = pfnGetPlayerUserId;
  memcpy (pengfuncsFromEngine, &meta_engfuncs, sizeof (enginefuncs_t));
  return TRUE;
}


/////////////////////////////////////////////////
//
// DLL ENTRY POINT AND SUCH / IMPORTANT HOOKING
// STUFF GOES ON HERE
//
/////////////////////////////////////////////////

// Required DLL entry point
bool WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
    ITM_Log("==============================\n");
    ITM_Log("DLL: DLLMain()\n");
    ITM_Log("==============================\n");

   if (fdwReason == DLL_PROCESS_ATTACH)
   {
       // DLL attached
        ITM_Log("DLL: Attaching\n");
   }
   else if (fdwReason == DLL_PROCESS_DETACH)
   {
       // DLL detached
        ITM_Log("DLL: Detaching\n");
       // * free up any memory here * //
   }
   
   // METAMOD DOES NOT CARE ABOUT THIS ;)


   return TRUE;
}

// 
C_DLLEXPORT void WINAPI GiveFnptrsToDll (enginefuncs_t * pengfuncsFromEngine, globalvars_t * pGlobals)
{
  ITM_Log("DLL: GiveFnptrsToDll\n");
  // get the engine functions from the engine...
  memcpy (&g_engfuncs, pengfuncsFromEngine, sizeof (enginefuncs_t));
  gpGlobals = pGlobals;  
  ITM_Log("DLL: Done\n");
}


//////////////////////////////////////////////////////////////
// Here the Half-Life 1 functions (hooked ones) are stored
// But instead of putting all the code there, i have choosen
// to run the proper IF_ global functions
//////////////////////////////////////////////////////////////
void GameDLLInit (void)
{
    IF_Init();  // Run the INTERFACE_INIT() function
    ITM_Log("ITM: GameDLLInit()\n");
    RETURN_META (MRES_IGNORED);
}


// Called on every spawned object
int Spawn (edict_t * pent)
{
    IF_Spawn(pent);
    RETURN_META_VALUE (MRES_IGNORED, 0);
}

// client connects
BOOL ClientConnect (edict_t * pEntity, const char *pszName, const char *pszAddress, char szRejectReason[128])
{
  if (gpGlobals->deathmatch)
  {
      int count = 0;

      // check if this client is the listen server client
      if (strcmp (pszAddress, "loopback") == 0)
	  {
		  // save the edict of the listen server client...
		  pHostEdict = pEntity;
	  }

      // check if this is NOT a bot joining the server...
      if (strcmp (pszAddress, "127.0.0.1") != 0)
	  {
		  // don't try to add bots for 60 seconds, give client time to get added
		  //bot_check_time = gpGlobals->time + 60.0;
	  }
      else
	  {

	  }
  }

  RETURN_META_VALUE (MRES_IGNORED, 0);
}

// On disconnect
void ClientDisconnect (edict_t * pEntity)
{
  if (gpGlobals->deathmatch)
  {
      int i;

      i = 0;

      while ((i < 32) && (clients[i] != pEntity))
          i++;

      // Find client and remove it from the list
      if (i < 32)
          clients[i] = NULL;

	  // when a bot...
      for (i = 0; i < 32; i++)
	  {
          // remove bot
          if (ITM_Bots[i].iIndex == i)
		  {
			  // someone kicked this bot off of the server...
			  ITM_Bots[i].bUsed = false;	// this slot is now free to use
			  //bots[i].fKickTime = gpGlobals->time;	// save the kicked time
			  //bots[i].pEdict = NULL; // make NULL
			  break;
		  }
	  }
  }

  RETURN_META (MRES_IGNORED);
}



// Client is put in the server
void ClientPutInServer (edict_t * pEntity)
{
  int i = 0;

  while ((i < 32) && (clients[i] != NULL))
    i++;

  if (i < 32)
    clients[i] = pEntity;	// store this clients edict in the clients array

  RETURN_META (MRES_IGNORED);
}


// Clientcommand
void ClientCommand (edict_t * pEntity)
{
    // NOT USED
    RETURN_META (MRES_IGNORED);
}

// Updating sequence , aka StartFrame in HL1
void StartFrame (void)
{
  IF_UpdateSequence();              // run this one
  RETURN_META (MRES_IGNORED);
}



////////////////////////////////////////////////////////////////
// From here the IF_ functions begin
////////////////////////////////////////////////////////////////
void IF_Init()
{

    int iIndex=0;
    for (iIndex=0; iIndex < 32; iIndex++)
        clients[iIndex]=NULL;
        
    SERVER_PRINT ("ITMPLUGIN: Initialized\n");
}

// Called on 'worldspawn'
void IF_Spawn(edict_t *pent)
{
    // Init game rules
    if (gpGlobals->deathmatch)
    {

      char *pClassname = (char *) STRING (pent->v.classname);

      // On worldspawn , level initialization
      if (strcmp (pClassname, "worldspawn") == 0)
	  {
		  
	  }	  
    }
}


// SERVER Command: ITM
void ITM_ServerCommand (void)
{
  const char *pcmd = CMD_ARGV (1);
  const char *arg1 = CMD_ARGV (2);
  const char *arg2 = CMD_ARGV (3);
  const char *arg3 = CMD_ARGV (4);
  const char *arg4 = CMD_ARGV (5);

  // When we need an edict to send messages to, we do that to the pHostEdict
  edict_t *pEntity = pHostEdict;

  // Handle command here
  if (FStrEq (pcmd, "hello"))
  {
    SERVER_PRINT("ITM SAYS: HELLO WORLD\n");
  }

  // Spawn a bot
  if (FStrEq (pcmd, "spawn"))
  {
     
  }  
      
  return;
}

// Spawn post
int Spawn_Post (edict_t *pent)
{
   // solves the bots unable to see through certain types of glass bug.
   // MAPPERS: NEVER ALLOW A TRANSPARENT ENTITY TO WEAR THE FL_WORLDBRUSH FLAG !!!

   // is this a transparent entity ? 
   if (pent->v.rendermode == kRenderTransTexture)
	   pent->v.flags &= ~FL_WORLDBRUSH; // then clear the FL_WORLDBRUSH flag   

RETURN_META_VALUE (MRES_IGNORED, 0);
} 


C_DLLEXPORT int GetEntityAPI2 (DLL_FUNCTIONS * pFunctionTable, int *interfaceVersion)
{
  gFunctionTable.pfnGameInit = GameDLLInit;
  gFunctionTable.pfnSpawn = Spawn;
  gFunctionTable.pfnClientConnect = ClientConnect;
  gFunctionTable.pfnClientDisconnect = ClientDisconnect;
  gFunctionTable.pfnClientPutInServer = ClientPutInServer;
  gFunctionTable.pfnClientCommand = ClientCommand;
  gFunctionTable.pfnStartFrame = StartFrame;
  memcpy (pFunctionTable, &gFunctionTable, sizeof (DLL_FUNCTIONS));
  return (TRUE);
}

C_DLLEXPORT int GetEntityAPI2_Post (DLL_FUNCTIONS * pFunctionTable, int *interfaceVersion) 
{ 
  gFunctionTable_post.pfnSpawn = Spawn_Post; // need to declare another gFunctionTable_post in the top of the dll.cpp file 
  memcpy (pFunctionTable, &gFunctionTable_post, sizeof (DLL_FUNCTIONS));
  return (TRUE); 
}

/////////////////////////////////////////////////
//
// UPDATING SEQUENCE !!
//
/////////////////////////////////////////////////
//
// Purpose:
//
// 1. Update our entity data
// 2. Update player data
// 3. run the bot think functions
//
void IF_UpdateSequence()
{
    if (gpGlobals->deathmatch)
    {
//      edict_t *pPlayer;
      static float check_server_cmd = 0.0;
      static int i, index, player_index, bot_index;
      static float previous_time = -1.0;
      static float client_update_time = 0.0;
      clientdata_s cd;
      int count = 0;
      int waits = 0;		// How many bots had to wait.

      // if a new map has started then (MUST BE FIRST IN StartFrame)...
      if ((gpGlobals->time + 0.1) < previous_time)
	  {
		  check_server_cmd = 0.0;	// reset at start of map

		  count = 0;

		  // mark the bots as needing to be respawned...
          /*
		  for (index = 0; index < MAX_HL1_CLIENTS; index++)
		  {
			  if (count >= prev_num_bots)
			  {
				  bots[index].bIsUsed = false;
				  bots[index].respawn_state = RESPAWN_NONE;
				  bots[index].fKickTime = 0.0;
			  }

			  if (bots[index].bIsUsed)	// is this slot used?
			  {
				  bots[index].respawn_state = RESPAWN_NEED_TO_RESPAWN;
				  count++;
			  }

			  // check for any bots that were very recently kicked...
			  if ((bots[index].fKickTime + 5.0) > previous_time)
			  {
				  bots[index].respawn_state = RESPAWN_NEED_TO_RESPAWN;
				  count++;
			  }
			  else
				  bots[index].fKickTime = 0.0;	// reset to prevent false spawns later

			  // set the respawn time
			  if (IS_DEDICATED_SERVER ())
				  respawn_time = gpGlobals->time + 5.0;
			  else
				  respawn_time = gpGlobals->time + 20.0;

			  // Send welcome message
			  welcome_sent = false;
			  welcome_time = 0.0;
		  }*/

		  //ChatEngine.fThinkTimer = gpGlobals->time;
		  client_update_time = gpGlobals->time + 10.0;	// start updating client data again
		  bot_check_time = gpGlobals->time + 30.0;
	  }

      welcome_sent=false;
      welcome_time=0.0;

      //
      // SEND WELCOME MESSAGE 
      //
      if (welcome_sent == false)
	  {
		  int iIndex = 0;
		  if (welcome_time == 0.0)
		  {			  
			  // go through all clients (except bots)
			  for (iIndex = 1; iIndex <= gpGlobals->maxClients; iIndex++)
			  {
				  edict_t *pPlayer = INDEXENT (iIndex);
				  // skip invalid players
				  if ((pPlayer) && (!pPlayer->free))
				  {
					  // we found a player which is alive. w00t
					  welcome_time = gpGlobals->time + 10.0;
					  break;
				  }
			  }
		  }
		  
		  if ((welcome_time > 0.0) && (welcome_time < gpGlobals->time))
		  {
			  // let's send a welcome message to this client...                      
			  char total_welcome[256];
			  sprintf (total_welcome, "ITM-BotFrame\nBy Stefan Hendriks\n");
			  int r, g, b;

              for (iIndex = 1; iIndex <= gpGlobals->maxClients; iIndex++)
			  {
				  edict_t *pPlayer = INDEXENT (iIndex);
				  // skip invalid players and skip self (i.e. this bot)
				  if ((pPlayer) && (!pPlayer->free))
				  {
					  // skip bots!
					  if (UTIL_GetBotPointer(pPlayer))
						  continue;
					  
					  // skip fake clients
					  if ((pPlayer->v.flags & FL_FAKECLIENT))			
						  continue;
					  
					  // random color
					  r = RANDOM_LONG (30, 255);
					  g = RANDOM_LONG (30, 255);
					  b = RANDOM_LONG (30, 255);
					  HUD_DrawString (r, g, b, total_welcome, pPlayer);
				  }
			  }

			  welcome_sent = true;	// clear this so we only do it once
			  welcome_time = 0.0;
		  }
	  }

      // Update clients
      if (client_update_time <= gpGlobals->time)
	  {
		  client_update_time = gpGlobals->time + 1.0;

		  for (i = 0; i < MAX_HL1_CLIENTS; i++)
		  {
			  if (ITM_Bots[i].bUsed)
			  {
				  memset (&cd, 0, sizeof (cd));

                  // Use clients[i] !!! since we do not have a reference
                  // to the bot architecture.
				  MDLL_UpdateClientData (clients[i], 1, &cd);

                  /*
				  // see if a weapon was dropped...
				  if (bots[i].bot_weapons != cd.weapons)
				  {
					  bots[i].bot_weapons = cd.weapons;
				  }*/
			  }
		  }
	  }

	  count = 0;
    
      // -------------------------------------------------------------
      // GAME : Update game status first, before we go think about it
      // -------------------------------------------------------------
      //
      // UPDATE ENTITIES
      //
      //

      //
      // UPDATE PLAYERS
      //

      //
      // BOTS THINK
      //
      count = BOTS_THINK();

      // Keep number of bots up-to-date.
//      if (count > num_bots)
//		  num_bots = count;

      // are we currently respawning bots and is it time to spawn one yet?
      /*
      if ((respawn_time > 1.0) && (respawn_time <= gpGlobals->time))
	  {
		  int index = 0;
         
		  // find bot needing to be respawned...
		  while ((index < 32) && (bots[index].respawn_state != RESPAWN_NEED_TO_RESPAWN))
			  index++;

		  if (index < 32)
		  {
			  bots[index].respawn_state = RESPAWN_IS_RESPAWNING;
			  bots[index].bIsUsed = false;	// free up this slot

			  // respawn 1 bot then wait a while (otherwise engine crashes)
			  // 01/07/04 - Stefan - Thanks Evy for pointing out this one: On skill 10
			  // the c_skill (was [2]) would be messed up (perhaps this is some memory related bug  later on?)
			  char c_skill[3];
			  char c_team[2];
			  char c_class[3];

			  sprintf (c_skill, "%d", bots[index].bot_skill);
			  sprintf (c_team, "%d", bots[index].iTeam);
			  sprintf (c_class, "%d", bots[index].bot_class);

			  Game.CreateBot (NULL, c_team, c_skill, c_class,
				  bots[index].name);

			  // 01/07/04 - Stefan - make 100% sure we do not crash on this part with the auto-add function
			  f_minplayers_think = gpGlobals->time + 15; // do not check this for 15 seconds from now

			  respawn_time = gpGlobals->time + 2.0;	// set next respawn time
			  bot_check_time = gpGlobals->time + 5.0;
		  }
		  else
		  {
			  respawn_time = 0.0;
		  }
	  }

		  if (!IS_DEDICATED_SERVER () && !spawn_time_reset)
		  {
			  if (pHostEdict != NULL)
			  {
				  if (IsAlive (pHostEdict))
				  {
					  spawn_time_reset = TRUE;

					  if (respawn_time >= 1.0)
						  respawn_time = min (respawn_time, gpGlobals->time + (float) 1.0);

					  if (bot_cfg_pause_time >= 1.0)
						  bot_cfg_pause_time = min (bot_cfg_pause_time, gpGlobals->time + (float) 1.0);
				  }
			  }
		  }
		  
	  }
*/




      previous_time = gpGlobals->time;
      
    } // GPGLOBALS->DEATHMATCH
} // IF_UPDATESEQUENCE


/////////////////////////////////////////////////
//
// !!! FUNCTIONS THAT ARE OPTIONAL/GAME SPECIFIC !!!
//
/////////////////////////////////////////////////
c_ITM_Bot * UTIL_GetBotPointer (edict_t * pEdict)
{
    // 2 steps
    //
    // 1 find this client edict in the clients list
    // 
    // if found, then use the iIndex nr to find
    // this match for a bot iIndex and then return
    // that pointer

    int iIndex=0, iBotIndex=0;
    for (iIndex = 0; iIndex < MAX_HL1_CLIENTS; iIndex++)
    {
        if (clients[iIndex] == pEdict)
        {
            // Found the client, now find the bot 
            for (iBotIndex=0; iBotIndex < MAX_HL1_CLIENTS; iBotIndex++)
                if (ITM_Bots[iBotIndex].iIndex == iIndex)
                {
                    return &ITM_Bots[iBotIndex];
                }

            // when we come here, we did not succeed in finding the bot pointer
            SERVER_PRINT("ERROR: Could not find bot pointer\n");
            break;
        }
    }

    if (iIndex >= MAX_HL1_CLIENTS)
        SERVER_PRINT("ERROR: Could not find client edict\n");
        
  return NULL;			// return NULL if edict is not a bot
}
 
// Returns BOT index based on pEdict
int UTIL_GetBotIndex (edict_t * pEdict)
{
    int iIndex=0, iBotIndex=0;
    for (iIndex = 0; iIndex < MAX_HL1_CLIENTS; iIndex++)
    {
        if (clients[iIndex] == pEdict)
        {
            // Found the client, now find the bot 
            for (iBotIndex=0; iBotIndex < MAX_HL1_CLIENTS; iBotIndex++)
                if (ITM_Bots[iBotIndex].iIndex == iIndex)
                {
                    return iBotIndex;
                }
        }
    }
 /* int index;

  for (index = 0; index < 32; index++)
    {
      if (bots[index].pEdict == pEdict)
	{
	  return index;
	}
    }
    */

  return -1;			// return -1 if edict is not a bot
}
 

// Using POD/SDK source to print nice messages on the client machine
void HUD_DrawString (int r, int g, int b, char *msg, edict_t * edict)
{
  // FROM PODBOT SOURCE
  // Hacked together Version of HUD_DrawString
  MESSAGE_BEGIN (MSG_ONE, SVC_TEMPENTITY, NULL, edict);
  WRITE_BYTE (TE_TEXTMESSAGE);
  WRITE_BYTE (1);
  WRITE_SHORT (FixedSigned16 (-1, 1 << 13));
  WRITE_SHORT (FixedSigned16 (0, 1 << 13));
  WRITE_BYTE (2);
  WRITE_BYTE (r);		//r
  WRITE_BYTE (g);		//g
  WRITE_BYTE (b);		//b
  WRITE_BYTE (0);
  WRITE_BYTE (255);
  WRITE_BYTE (255);
  WRITE_BYTE (255);
  WRITE_BYTE (200);
  WRITE_SHORT (FixedUnsigned16 (0.0078125, 1 << 8));
  WRITE_SHORT (FixedUnsigned16 (2, 1 << 8));
  WRITE_SHORT (FixedUnsigned16 (6, 1 << 8));
  WRITE_SHORT (FixedUnsigned16 (0.1, 1 << 8));
  WRITE_STRING ((const char *) &msg[0]);
  MESSAGE_END ();
}

 

/////////////////////////////////////////////////
//
// !!! FUNCTIONS THAT CALL BACK ON INFO LAYER !!!
//
/////////////////////////////////////////////////
int IF_MakeItHappen(c_ITM_Bot *pBot)
{
    // 
    // runplayermove

    return 0;       // success
}


///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
//
// SDK_UTIL.CPP CONTENTS OF METAMOD BELOW
//
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////



// vi: set ts=4 sw=4 :
// vim: set tw=75 :

// sdk_util.cpp - utility routines from HL SDK util.cpp

// Selected portions of dlls/util.cpp from SDK 2.1.
// Functions copied from there as needed...
// And modified to avoid buffer overflows (argh).
// Also modified to remove dependency on CBaseEntity class.

/***
*
*	Copyright (c) 1999, 2000 Valve LLC. All rights reserved.
*	
*	This product contains software technology licensed from Id 
*	Software, Inc. ("Id Technology").  Id Technology (c) 1996 Id Software, Inc. 
*	All Rights Reserved.
*
*   Use, distribution, and modification of this source code and/or resulting
*   object code is restricted to non-commercial enhancements to products from
*   Valve LLC.  All other use, distribution, or modification is prohibited
*   without written permission from Valve LLC.
*
****/
/*

===== util.cpp ========================================================

  Utility code.  Really not optional after all.

*/

#include "osdep.h"			// win32 vsnprintf, etc

char* UTIL_VarArgs( char *format, ... )
{
	va_list		argptr;
	static char		string[1024];
	
	va_start (argptr, format);
	vsnprintf (string, sizeof(string), format, argptr);
	va_end (argptr);

	return string;
}
	

//=========================================================
// UTIL_LogPrintf - Prints a logged message to console.
// Preceded by LOG: ( timestamp ) < message >
//=========================================================
void UTIL_LogPrintf( char *fmt, ... )
{
	va_list			argptr;
	static char		string[1024];
	
	va_start ( argptr, fmt );
	vsnprintf ( string, sizeof(string), fmt, argptr );
	va_end   ( argptr );

	// Print to server console
	ALERT( at_logged, "%s", string );
}


void META_UTIL_HudMessage(edict_t *pEntity, const hudtextparms_t &textparms, 
		const char *pMessage)
{
	if ( FNullEnt(pEntity) || pEntity->free )
		return;

	MESSAGE_BEGIN( MSG_ONE, SVC_TEMPENTITY, NULL, pEntity );
		WRITE_BYTE( TE_TEXTMESSAGE );
		WRITE_BYTE( textparms.channel & 0xFF );

		WRITE_SHORT( FixedSigned16( textparms.x, 1<<13 ) );
		WRITE_SHORT( FixedSigned16( textparms.y, 1<<13 ) );
		WRITE_BYTE( textparms.effect );

		WRITE_BYTE( textparms.r1 );
		WRITE_BYTE( textparms.g1 );
		WRITE_BYTE( textparms.b1 );
		WRITE_BYTE( textparms.a1 );

		WRITE_BYTE( textparms.r2 );
		WRITE_BYTE( textparms.g2 );
		WRITE_BYTE( textparms.b2 );
		WRITE_BYTE( textparms.a2 );

		WRITE_SHORT( FixedUnsigned16( textparms.fadeinTime, 1<<8 ) );
		WRITE_SHORT( FixedUnsigned16( textparms.fadeoutTime, 1<<8 ) );
		WRITE_SHORT( FixedUnsigned16( textparms.holdTime, 1<<8 ) );

		if ( textparms.effect == 2 )
			WRITE_SHORT( FixedUnsigned16( textparms.fxTime, 1<<8 ) );
		
		if ( strlen( pMessage ) < 512 )
		{
			WRITE_STRING( pMessage );
		}
		else
		{
			char tmp[512];
			strncpy( tmp, pMessage, 511 );
			tmp[511] = 0;
			WRITE_STRING( tmp );
		}
	MESSAGE_END();
}

short FixedSigned16( float value, float scale )
{
	int output;

	output = (int) (value * scale);

	if ( output > 32767 )
		output = 32767;

	if ( output < -32768 )
		output = -32768;

	return (short)output;
}

unsigned short FixedUnsigned16( float value, float scale )
{
	int output;

	output = (int) (value * scale);
	if ( output < 0 )
		output = 0;
	if ( output > 0xFFFF )
		output = 0xFFFF;

	return (unsigned short)output;
}
 


///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
//
// BOT_CLIENT.CPP CONTENTS BELOW
//
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
// This message is sent when the Counter-Strike VGUI menu is displayed.
void
BotClient_CS_VGUI (void *p, int bot_index)
{/*
  //DebugOut("bot_client: BotClient_CS_VGUI()\n");
  if ((*(int *) p) == 2)	// is it a team select menu?

    bots[bot_index].start_action = MSG_CS_TEAM_SELECT;

  else if ((*(int *) p) == 26)	// is is a terrorist model select menu?

    bots[bot_index].start_action = MSG_CS_T_SELECT;

  else if ((*(int *) p) == 27)	// is is a counter-terrorist model select menu?

    bots[bot_index].start_action = MSG_CS_CT_SELECT;*/
}

// This message is sent when a menu is being displayed in Counter-Strike.
void BotClient_CS_ShowMenu (void *p, int bot_index)
{
  //DebugOut("bot_client: BotClient_CS_ShowMenu()\n");
  static int state = 0;		// current state machine state

  if (state < 3)
    {
      state++;			// ignore first 3 fields of message
      return;
    }

  if (strcmp ((char *) p, "#Team_Select") == 0)	// team select menu?
    {
   //   bots[bot_index].start_action = MSG_CS_TEAM_SELECT;
    }
  else if (strcmp ((char *) p, "#Terrorist_Select") == 0)	// T model select?
    {
 //     bots[bot_index].start_action = MSG_CS_T_SELECT;
    }
  else if (strcmp ((char *) p, "#CT_Select") == 0)	// CT model select menu?
    {
    //  bots[bot_index].start_action = MSG_CS_CT_SELECT;
    }

  state = 0;			// reset state machine
}

// This message is sent when a client joins the game.  All of the weapons
// are sent with the weapon ID and information about what ammo is used.
void BotClient_Valve_WeaponList (void *p, int bot_index)
{
  //DebugOut("bot_client (FINDME): BotClient_Valve_WeaponList()\n");
  /*static int state = 0;		// current state machine state
  static bot_weapon_t bot_weapon;

  if (state == 0)
    {
      state++;
      strcpy (bot_weapon.szClassname, (char *) p);
    }
  else if (state == 1)
    {
      state++;
      bot_weapon.iAmmo1 = *(int *) p;	// ammo index 1
    }
  else if (state == 2)
    {
      state++;
      bot_weapon.iAmmo1Max = *(int *) p;	// max ammo1
    }
  else if (state == 3)
    {
      state++;
      bot_weapon.iAmmo2 = *(int *) p;	// ammo index 2
    }
  else if (state == 4)
    {
      state++;
      bot_weapon.iAmmo2Max = *(int *) p;	// max ammo2
    }
  else if (state == 5)
    {
      state++;
      bot_weapon.iSlot = *(int *) p;	// slot for this weapon
    }
  else if (state == 6)
    {
      state++;
      bot_weapon.iPosition = *(int *) p;	// position in slot
    }
  else if (state == 7)
    {
      state++;
      bot_weapon.iId = *(int *) p;	// weapon ID                      
    }
  else if (state == 8)
    {
      state = 0;

      bot_weapon.iFlags = *(int *) p;	// flags for weapon (WTF???)

      // store away this weapon with it's ammo information...
      weapon_defs[bot_weapon.iId] = bot_weapon;

      /*
         fp=fopen("!BotWeapon.txt","a");       
         fprintf(fp,"[%s]\n", bot_weapon.szClassname);      
         fprintf(fp, "; COUNTER-STRIKE SPECIFIC INFORMATION, DO NOT EDIT!\n");
         fprintf(fp,"Ammo1Index=%d\n", bot_weapon.iAmmo1);
         fprintf(fp,"Ammo1Max=%d\n", bot_weapon.iAmmo1Max);
         fprintf(fp,"Ammo2Index=%d\n", bot_weapon.iAmmo2);
         fprintf(fp,"Ammo2Max=%d\n", bot_weapon.iAmmo2Max);
         fprintf(fp,"Slot=%d\n", bot_weapon.iSlot);
         fprintf(fp,"iPosition=%d\n", bot_weapon.iPosition);
         fprintf(fp, "; --- END\n");
         fclose(fp);    
       */
    //}
}

void BotClient_CS_WeaponList (void *p, int bot_index)
{
  //DebugOut("bot_client: BotClient_CS_WeaponList()\n");
  // this is just like the Valve Weapon List message
  BotClient_Valve_WeaponList (p, bot_index);
}

// This message is sent when a weapon is selected (either by the bot chosing
// a weapon or by the server auto assigning the bot a weapon).
void BotClient_Valve_CurrentWeapon (void *p, int bot_index)
{
    /*
  static int state = 0;		// current state machine state
  static int iState;
  static int iId;
  static int iClip;

  if (state == 0)
    {
      state++;
      iState = *(int *) p;	// state of the current weapon
    }
  else if (state == 1)
    {
      state++;
      iId = *(int *) p;		// weapon ID of current weapon
    }
  else if (state == 2)
    {
      state = 0;

      iClip = *(int *) p;	// ammo currently in the clip for this weapon

      if (iId <= 31)
	{
	  bots[bot_index].bot_weapons |= (1 << iId);	// set this weapon bit

	  if (iState == 1)
	    {
	      bots[bot_index].current_weapon.iId = iId;	// weapon id
	      bots[bot_index].current_weapon.iClip = iClip;	// currently in clp

	      // update the ammo counts for this weapon...

	      bots[bot_index].current_weapon.iAmmo1 =
		bots[bot_index].m_rgAmmo[weapon_defs[iId].iAmmo1];

	      bots[bot_index].current_weapon.iAmmo2 =
		bots[bot_index].m_rgAmmo[weapon_defs[iId].iAmmo2];

	    }
	}
    }*/

}

void BotClient_CS_CurrentWeapon (void *p, int bot_index)
{
  //DebugOut("bot_client: BotClient_CS_CurrentWeapon()\n");
  // this is just like the Valve Current Weapon message
  BotClient_Valve_CurrentWeapon (p, bot_index);
}

// This message is sent whenever ammo ammounts are adjusted (up or down).
void BotClient_Valve_AmmoX (void *p, int bot_index)
{
    /*
  //DebugOut("bot_client: BotClient_Valve_AmmoX()\n");
  static int state = 0;		// current state machine state
  static int index;
  static int ammount;
  int ammo_index;

  if (state == 0)
    {
      state++;
      index = *(int *) p;	// ammo index (for type of ammo)
    }
  else if (state == 1)
    {
      state = 0;

      ammount = *(int *) p;	// the ammount of ammo currently available

      bots[bot_index].m_rgAmmo[index] = ammount;	// store it away

      ammo_index = bots[bot_index].current_weapon.iId;

      // update the ammo counts for this weapon...
      bots[bot_index].current_weapon.iAmmo1 =
	bots[bot_index].m_rgAmmo[weapon_defs[ammo_index].iAmmo1];

      bots[bot_index].current_weapon.iAmmo2 =
	bots[bot_index].m_rgAmmo[weapon_defs[ammo_index].iAmmo2];

    }*/
}

void BotClient_CS_AmmoX (void *p, int bot_index)
{
  //DebugOut("bot_client: BotClient_CS_AmmoX()\n");
  // this is just like the Valve AmmoX message
  BotClient_Valve_AmmoX (p, bot_index);
}


// This message is sent when the bot picks up some ammo (AmmoX messages are
// also sent so this message is probably not really necessary except it
// allows the HUD to draw pictures of ammo that have been picked up.  The
// bots don't really need pictures since they don't have any eyes anyway.
void BotClient_Valve_AmmoPickup (void *p, int bot_index)
{
    /*
  //DebugOut("bot_client: BotClient_Valve_AmmoPickup()\n");
  static int state = 0;		// current state machine state
  static int index;
  static int ammount;
  int ammo_index;

  if (state == 0)
    {
      state++;
      index = *(int *) p;
    }
  else if (state == 1)
    {
      state = 0;

      ammount = *(int *) p;

      bots[bot_index].m_rgAmmo[index] = ammount;

      ammo_index = bots[bot_index].current_weapon.iId;

      // update the ammo counts for this weapon...
      bots[bot_index].current_weapon.iAmmo1 =
	bots[bot_index].m_rgAmmo[weapon_defs[ammo_index].iAmmo1];
      bots[bot_index].current_weapon.iAmmo2 =
	bots[bot_index].m_rgAmmo[weapon_defs[ammo_index].iAmmo2];
    }*/
}

void BotClient_CS_AmmoPickup (void *p, int bot_index)
{
  //DebugOut("bot_client: BotClient_CS_AmmoPickup()\n");
  // this is just like the Valve Ammo Pickup message
  BotClient_Valve_AmmoPickup (p, bot_index);
}

// This message gets sent when the bot picks up a weapon.
void BotClient_Valve_WeaponPickup (void *p, int bot_index)
{
    /*
  //DebugOut("bot_client: BotClient_Valve_WeaponPickup()\n");
  int index;

  index = *(int *) p;

  // set this weapon bit to indicate that we are carrying this weapon
  bots[bot_index].bot_weapons |= (1 << index); */
}

void BotClient_CS_WeaponPickup (void *p, int bot_index)
{
  //DebugOut("bot_client: BotClient_CS_WeaponPickup()\n");
  // this is just like the Valve Weapon Pickup message
  BotClient_Valve_WeaponPickup (p, bot_index);
}

// This message gets sent when the bot picks up an item (like a battery
// or a healthkit)
void BotClient_Valve_ItemPickup (void *p, int bot_index)
{
  //DebugOut("bot_client: BotClient_Valve_ItemPickup()\n");
}

void BotClient_CS_ItemPickup (void *p, int bot_index)
{
  //DebugOut("bot_client: BotClient_CS_ItemPickup()\n");
  // this is just like the Valve Item Pickup message
  BotClient_Valve_ItemPickup (p, bot_index);
}

// This message gets sent when the bots health changes.
void BotClient_Valve_Health (void *p, int bot_index)
{
  //DebugOut("bot_client: BotClient_Valve_Health()\n");
  //bots[bot_index].bot_health = *(int *) p;	// health ammount

   
    // CAN BE REMOVED, UPDATED BY UPDATESEQUENCE
}

void BotClient_CS_Health (void *p, int bot_index)
{
  //DebugOut("bot_client: BotClient_CS_Health()\n");
  // this is just like the Valve Health message
  BotClient_Valve_Health (p, bot_index);
  // CAN BE REMOVED, UPDATED BY UPDATESEQUENCE
}

// This message gets sent when the bots armor changes.
void BotClient_Valve_Battery (void *p, int bot_index)
{
  //DebugOut("bot_client: BotClient_Valve_Battery()\n");
  //bots[bot_index].bot_armor = *(int *) p;	// armor ammount
  // CAN BE REMOVED, UPDATED BY UPDATESEQUENCE
}

void
BotClient_CS_Battery (void *p, int bot_index)
{
  //DebugOut("bot_client: BotClient_CS_Battery()\n");
  // this is just like the Valve Battery message
  //BotClient_Valve_Battery (p, bot_index);
    // CAN BE REMOVED, UPDATED BY UPDATESEQUENCE
}

// This message gets sent when the bots are getting damaged.
void BotClient_Valve_Damage (void *p, int bot_index)
{/*
  //DebugOut("bot_client: BotClient_Valve_Damage()\n");
  static int state = 0;		// current state machine state
  static int damage_armor;
  static int damage_taken;
  static int damage_bits;	// type of damage being done
  static Vector damage_origin;

  if (state == 0)
    {
      state++;
      damage_armor = *(int *) p;
    }
  else if (state == 1)
    {
      state++;
      damage_taken = *(int *) p;
    }
  else if (state == 2)
    {
      state++;
      damage_bits = *(int *) p;
    }
  else if (state == 3)
    {
      state++;
      damage_origin.x = *(float *) p;
    }
  else if (state == 4)
    {
      state++;
      damage_origin.y = *(float *) p;
    }
  else if (state == 5)
    {
      state = 0;

      damage_origin.z = *(float *) p;
      if ((damage_armor > 0) || (damage_taken > 0))
	{

	  // Damage recieved:
	  // - when the prev node was higher (so we are sure we do FIX the correct nodes!)
	  //   we fix it (only when dist > 90)
	  cBot *pBot = &bots[bot_index];
	  if (damage_bits & (DMG_FALL | DMG_CRUSH))

	    {
	      LOG_CONSOLE (PLID, "realbot - fixed connection with falling.");
	      LOG_MESSAGE (PLID, "realbot - fixed connection with falling.");
	      if (pBot->bot_pathid > 0)

		{
		  int iNode = NodeMachine.NodeFromPath (pBot->iIndex,
							(pBot->bot_pathid -
							 1));
		  float fDist =
		    fabs (damage_origin.z -
			  NodeMachine.node_vector (iNode).z);
		  if (fDist > 90)

		    {

		      // we know where we came from, and we know where we went to
		      int iNodeTo = NodeMachine.NodeFromPath (pBot->iIndex,
							      pBot->
							      bot_pathid);

		      // remove connection
		      NodeMachine.remove_neighbour_node (iNode, iNodeTo);
		    }
		}
	    }

	  // ignore certain types of damage...
	  if (damage_bits & IGNORE_DAMAGE)
	    return;

	  // depending on bot skill slow this bot down a bit
	  pBot->f_move_speed *= (((10 - pBot->bot_skill) + 1) / 10);

	  // if the bot doesn't have an enemy and someone is shooting at it then
	  // turn in the attacker's direction...
	  if (bots[bot_index].pBotEnemy == NULL)
	    {
	      // face the attacker...

	      // Face danger vector
	      pBot->vHead = damage_origin;
	      pBot->vBody = damage_origin;

	      // move to damage vector
	      pBot->f_camp_time = gpGlobals->time;	// stop camping
	      pBot->iGoalNode = NodeMachine.close (damage_origin, 150, NULL);
	      pBot->bot_pathid = -1;

	      ///////////// END STEFAN
	    }
	}
    }*/

// CAN BE REMOVED, UPDATED BY UPDATESEQUENCE
// CAN BE REMOVED, UPDATED BY UPDATESEQUENCE
// CAN BE REMOVED, UPDATED BY UPDATESEQUENCE
// CAN BE REMOVED, UPDATED BY UPDATESEQUENCE
}

void BotClient_CS_Damage (void *p, int bot_index)
{
  //DebugOut("bot_client: BotClient_CS_Damage()\n");
  // this is just like the Valve Battery message
  BotClient_Valve_Damage (p, bot_index);
}


void BotClient_CS_SayText(void *p, int bot_index)
{/*
	static unsigned char ucEntIndex;

	static int state = 0;		// current state machine state

	// Different Counter-Strike versions mean different
	// handling of this "SayText" thingy.
	if (counterstrike==0)
	{
		if (state == 0)
		{
			ucEntIndex = *(unsigned char *)p;
		}
		else if (state == 1)
		{		
			cBot *pBot=&bots[bot_index];
			
			if (ENTINDEX(pBot->pEdict) != ucEntIndex)
			{				
				char sentence[128];
				char chSentence[128];
				char netname[30];			

				memset (sentence, 0, sizeof (sentence));
				memset (chSentence, 0, sizeof (chSentence));
				memset (netname, 0, sizeof (netname));
								
				strcpy(sentence,(char *)p);
											
				int length=0;

				// FIXED: In any case that this might return NULL, do not crash the server
				if (strstr (sentence, " : "))
					length = strlen (sentence) - strlen (strstr (sentence, " : "));

				
				int tc=0,c;

				for (c=length; c < 128; c++)
				{
					chSentence[tc] = sentence[c];
					tc++;
				}
/*
				int nc=0; 
					c=2;

				if (strstr(sentence, "*DEAD*"))
					c+=6;
				
				if (counterstrike == 0)
					c=1;

				for (c; c < length; c++)
				{
					netname[nc] = sentence[c];
					nc++;
				}
*/

    /*
				edict_t *pPlayer = INDEXENT (ucEntIndex);
				strcpy(netname, STRING(pPlayer->v.netname));

				ChatEngine.set_sentence(netname, chSentence);
				state = -1;
			}
		}
	}
	else if (counterstrike==1)
	{
		// CS 1.6
		if (state == 0)
		{
			// who sent this message?
			ucEntIndex = *(unsigned char *)p;
		}
		// to who?
		else if (state == 1)
		{
			// here must be some team check so we do not let bots
			// of the other team react to this somehow.. 
			// - after playing with the dll i did not notice any weird stuff yet...
			// - so only when people discover some bugs with this we are going to fix this
			// - thing... ie "Only fix it when its broken".		
		}
		// don't know?
		else if (state == 2)
		{
			// do nothing here
		}
		// the actual sentence
		else if (state == 3)
		{
			// sentence
			char sentence[128];
			char netname[30];			
				
			// init
			memset (sentence, 0, sizeof (sentence));
			memset (netname, 0, sizeof (netname));
			
			// copy in memory
			strcpy(sentence,(char *)p);

			// copy netname
			edict_t *pPlayer = INDEXENT (ucEntIndex);
			strcpy(netname, STRING(pPlayer->v.netname));
			
			// and give chatengine something to do
			ChatEngine.set_sentence(netname, sentence);

			state=-1;
		}
	}
	state++;*/
}

// -------------------------------------------------------------
// BERKED -- call with nullptr to set state to zero.
// Save HUD display of bomb plantability.
// Would like to get rid of state.
// Converted to use switches.

// This message gets sent when the bot enters a buyzone
void BotClient_CS_StatusIcon (void *p, int bot_index)
{
  /*
     FROM SDK 2.3
     // Message handler for StatusIcon message
     // accepts five values:
     //         byte   : TRUE = ENABLE icon, FALSE = DISABLE icon
     //         string : the sprite name to display
     //         byte   : red
     //         byte   : green
     //         byte   : blue
   */

    /*
  static int state = 0;		// current state machine state
  static int EnableIcon;

  if (p == 0)			// A bandaid.  Make this whole thing more robust!
    {
      state = 0;
      return;
    }

  switch (state++)
    {
    case 0:			// Enable or not?
      EnableIcon = *(int *) p;	// check the byte
      break;
    case 1:			// Which icon
      if (strcmp ((char *) p, "buyzone") == 0)
	{
	  switch (EnableIcon)
	    {
	    case 0:		// Not in buy zone
	      state = 0;
	      break;
	    case 1:		// In buy zone
	      break;
	    default:
	      break;
	    }
	}
      else if (strcmp ((char *) p, "c4") == 0)
	{
	  switch (EnableIcon)
	    {
	    case 0:		// No C4
	      bots[bot_index].bHUD_C4_plantable = false;
	      state = 0;
	      break;
	    case 1:		// C4
	      bots[bot_index].bHUD_C4_plantable = false;
	      break;
	    case 2:		// C4 plantable
	      bots[bot_index].bHUD_C4_plantable = true;	// BINGO!!! -- BERKED
	      break;
	    default:
	      break;
	    }
	}
      else if (strcmp ((char *) p, "defuser") == 0)
	{
	  switch (EnableIcon)
	    {
	    case 0:		// No defuser
	      state = 0;
	      break;
	    case 1:		// defuser
	      break;
	    default:
	      break;
	    }
	}
      else if (strcmp ((char *) p, "rescue") == 0)
	{
	  switch (EnableIcon)
	    {
	    case 0:		// Not in rescue zone
	      state = 0;
	      break;
	    case 1:		// In rescue zone
	      break;
	    default:
	      break;
	    }
	}
      else			// something we don't know about yet.
	{
	}
      break;
    case 2:			// colors
      // RED
      break;
    case 3:
      // GREEN
      break;
    case 4:
      // BLUE
      state = 0;
      break;
    default:
      state = 0;
      break;
    }*/
}

// This message gets sent when the bots money ammount changes (for CS)
void BotClient_CS_Money (void *p, int bot_index)
{
    /*
  //DebugOut("bot_client: BotClient_CS_Money()\n");
  static int state = 0;		// current state machine state

  if (state == 0)
    {
      state++;

      bots[bot_index].bot_money = *(int *) p;	// amount of money
    }
  else
    {
      state = 0;		// ingore this field
    }*/
}

// This message gets sent when the bots get killed
void BotClient_Valve_DeathMsg (void *p, int bot_index)
{
  //DebugOut("bot_client: BotClient_Valve_DeathMsg()\n");
    /*
  static int state = 0;		// current state machine state
  static int killer_index;
  static int victim_index;
  static edict_t *victim_edict;
  static int index;

  if (state == 0)
    {
      state++;
      killer_index = *(int *) p;	// ENTINDEX() of killer
    }
  else if (state == 1)
    {
      state++;
      victim_index = *(int *) p;	// ENTINDEX() of victim
    }
  else if (state == 2)
    {
      state = 0;

      victim_edict = INDEXENT (victim_index);

      index = UTIL_GetBotIndex (victim_edict);

      // is this message about a bot being killed?
      if (index != -1)
	{
	  if ((killer_index == 0) || (killer_index == victim_index))
	    {
	      // bot killed by world (worldspawn) or bot killed self...
	      bots[index].killer_edict = NULL;
	    }
	  else
	    {
	      // store edict of player that killed this bot...
	      bots[index].killer_edict = INDEXENT (killer_index);
	    }
	}
    }*/
}

void BotClient_CS_DeathMsg (void *p, int bot_index)
{
  //DebugOut("bot_client: BotClient_CS_DeathMsg()\n");
  // this is just like the Valve DeathMsg message
  BotClient_Valve_DeathMsg (p, bot_index);
}

void BotClient_Valve_ScreenFade (void *p, int bot_index)
{
    /*
  //DebugOut("bot_client: BotClient_Valve_ScreenFade()\n");
  static int state = 0;		// current state machine state
  static int duration;
  static int hold_time;
  static int fade_flags;
  float length;

  if (state == 0)
    {
      state++;
      duration = *(int *) p;
    }
  else if (state == 1)
    {
      state++;
      hold_time = *(int *) p;
    }
  else if (state == 2)
    {
      state++;
      fade_flags = *(int *) p;
    }
  else if (state == 6)
    {
      state = 0;

      length = (duration + hold_time) / 4096.0;
      int iDevide = bots[bot_index].bot_skill;
      if (iDevide < 1)
		  iDevide = 1;

      length -= ((10 / iDevide) * 0.5);

      bots[bot_index].f_blinded_time = gpGlobals->time + length;

      // Get pointer and do some radio stuff here - Added by Stefan      
	  cBot *pBot;
	  pBot = &bots[bot_index];
      
	  int iCurrentNode=NodeMachine.close(pBot->pEdict->v.origin, NODE_ZONE, pBot->pEdict);
	  int iCoverNode=NodeMachine.node_cover(iCurrentNode, iCurrentNode, pBot->pEdict);

	  if (iCoverNode > -1)
	  {
		pBot->bot_pathid = -1;
		pBot->iGoalNode = iCoverNode;
	  }

    }
  else
    {
      state++;
    }*/
}

void BotClient_CS_ScreenFade (void *p, int bot_index)
{
  //DebugOut("bot_client: BotClient_CS_ScreenFade()\n");
  // this is just like the Valve ScreenFade message
  BotClient_Valve_ScreenFade (p, bot_index);
}

// STEFAN
// This message gets sent for HLTV, according to Alfred (Valve) also
// unique for each round-start.
void BotClient_CS_HLTV(void *p, int bot_index)
{/*
  static int state = 0;		// current state machine state
  static int players = 0;

  /* 
    From e-mail Alfred:

  // reset all players health for HLTV
	MESSAGE_BEGIN( MSG_SPEC, gmsgHLTV );
		WRITE_BYTE( 0 );	// 0 = all players
		WRITE_BYTE( 100 | 128 );
	MESSAGE_END();

	// reset all players FOV for HLTV
	MESSAGE_BEGIN( MSG_SPEC, gmsgHLTV );
		WRITE_BYTE( 0 );	// all players
		WRITE_BYTE( 0 );
	MESSAGE_END();

  */
/*
  if (state == 0)
  {
      players = *(int *) p;	// check the byte
      state++;
  }
  else if (state == 1)
  {
      // I check here on purpose. Multiple HLTV messages get sent within the game, 
      // by checking for the second state i AND the 'all players' flag as above in Alfreds
      // code i hopefully elminate all faulty 'new round' detections. Testing
      // has shown me that the state machine MUST be in, else you will get "New Round"
      // detections on strange occasions!

      // We could do some cool stuff here, but all i want to know is if the
      // round has (re)started, so i just set that flag and i am done dude.
      if (players == 0)
      {
          // New round started. CS 1.6
      }
      state = 0;
  }*/
}
// STEFAN - END 
